//
// file :         mercurythread.cpp
//
// description :  reading thread
//
// project :      TANGO Device Server
//
// $Author: graziano $ 
//
// $Revision: 1.10 $
//
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//

#include "mercurythread.h"

static const char __FILE__rev[] = __FILE__ " $Revision: 1.10 $";

//+------------------------------------------------------------------
//
//	method:			standathread::standathread()
//
//	description:	standathread constructor
//
//-------------------------------------------------------------------
mercurythread::mercurythread(PiMercury_ns::PiMercury *s):Tango::LogAdapter(s)
{ 
	DEBUG_STREAM << "mercurythread::mercurythread(): constructor... !" << endl;
	mymercury = s;
	mutex_sleep = new omni_mutex();
	
}

//+------------------------------------------------------------------
//
//	method:			standathread::~standathread()
//
//	description:	standathread destructor
//
//-------------------------------------------------------------------
mercurythread::~mercurythread()
{
	DEBUG_STREAM << "mercurythread::~mercurythread(): destructor... !" << endl;

	//mymercury->loopterminatedflag = 1;
	delete mutex_sleep;
	DEBUG_STREAM << "mercurythread::~mercurythread(): deleted... !" << endl;
}

//+------------------------------------------------------------------
//
//	method:			mercurythread::run()
//
//	description:	Run
//
//-------------------------------------------------------------------
void mercurythread::run(void *) 
{
	INFO_STREAM << "mercurythread::run(): running... !" << endl;	
	int pausesec,pausenano,
	    cnt_err = 0,
		max_cnt_err = 3,
		ret;
//	im540_map::iterator it;// = myim540->read_data.begin();
	
	string id;
	//update_scheduler();
	string resp;
	Tango::DevState stat_moving;
	mutex_sleep->lock();
	pausesec = pausesec_dfl = (int)(((double)(5.0 * mymercury->th_sleep))/1000);					//poll status slow when not moving
	pausenano = pausenano_dfl = (int)(((5.0 * mymercury->th_sleep)-pausesec_dfl*1000)*1000000);
	DEBUG_STREAM << "mercurythread::run(): pausesec=" << pausesec << " pausenano=" << pausenano << endl;
	mutex_sleep->unlock();
	int counter = 0;
	sleep(1);
	
	while (!mymercury->abortflag) 
	{	
		if(mymercury->rtmode)
		{
			ret = 0;
			if(mymercury->abort_rt_sleep)
				mymercury->abort_rt_sleep = false;
			mymercury->RTReadMercury();
#ifndef _SERVER_RT
			long tmp_val;
			struct timeval timestamp;
			unsigned long bunchnumber;
			int ret;
			ret = read_last(&tmp_val, &timestamp, &bunchnumber, mymercury->buf);
			*(mymercury->attr_Position_read) = (double)(tmp_val) / mymercury->positionRatio;
			DEBUG_STREAM << "mercurythread::run(): read_last returned: pos="<<tmp_val<<"="<<*(mymercury->attr_Position_read)<<" time s="<<(unsigned int)timestamp.tv_sec << " us="<<(unsigned int)timestamp.tv_usec<<" bunchnum="<<bunchnumber<< endl;
			Tango::TimeVal t_last;
			t_last.tv_sec = timestamp.tv_sec;
			t_last.tv_usec = timestamp.tv_usec;
			t_last.tv_nsec = 0;
			if(!mymercury->rtmode)
				continue;
			if(ret < 0)
			{
				string stat_tmp = mymercury->get_status();
				int pos = stat_tmp.find("| ERROR");
				if(pos > 0)
					stat_tmp = stat_tmp.substr(0,pos-1);
				mymercury->set_status(stat_tmp + " | ERROR BUFFER EMPTY");		
			}
			else if(mymercury->tv_compare(t_last,mymercury->last_rt_read) == 0)
			{
				string stat_tmp = mymercury->get_status();
				int pos = stat_tmp.find("| ERROR");
				if(pos > 0)
					stat_tmp = stat_tmp.substr(0,pos-1);
				mymercury->set_status(stat_tmp + " | ERROR BUFFER NOT UPDATING");
				DEBUG_STREAM << "mercurythread::run(): ERROR BUFFER NOT UPDATING t_last.tv_sec=" << t_last.tv_sec << " t_last.tv_usec=" << t_last.tv_usec << " last_rt_read.tv_sec=" << mymercury->last_rt_read.tv_sec << " last_rt_read.tv_usec=" << mymercury->last_rt_read.tv_usec << endl;		
			}
			else
			{
				string stat_tmp = mymercury->get_status();
				int pos = stat_tmp.find("| ERROR");
				if(pos > 0)
					mymercury->set_status(stat_tmp.substr(0,pos-1));
			}
			mymercury->last_rt_read = t_last;
#endif
		}
		/*else if(mymercury->init_homing)
		{
			mymercury->init_homing = false;
			homing();
		}*/
		else //if(!mymercury->rtmode)
		{
			ret = 0;
			//if(!mymercury->homing)
			{
				if(mymercury->WriteReadMercury(TELL_POSITION, "",resp, true) < 0)
				{
					WARN_STREAM << "mercurythread::run(): exception reading position: '" << resp << "'" << endl;
					ret = -1;
				}		
				else if(resp.length() > 2)
				{
					int pos;
					if(mymercury->c_86x_type < C_863_11)
					{
						if(sscanf(resp.c_str(), "P:%d",&pos) == 1)
						{
							*(mymercury->attr_Position_read) = (double)(pos) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read position: " << pos << endl;
						}
					}
					else
					{
						int ax;
						float dpos;
						if(sscanf(resp.c_str(), "%d=%f",&ax, &dpos) == 2)
						{
							*(mymercury->attr_Position_read) = (double)(dpos) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read position: " << dpos <<" ax="<<ax<<endl;
						}
					}
				}
				abort_sleep(((double)mymercury->wr_sleep)/1000);
				if(mymercury->c_86x_type == C_863_11 || mymercury->c_86x_type == C_863_12)
				{
					if(mymercury->WriteReadMercury(TELL_11_ERR, "",resp, true) < 0)
					{
						WARN_STREAM << "mercurythread::run(): exception reading position: '" << resp << "'" << endl;
						ret = -1;
					}		
					else if(resp.length() > 0)
					{
						strcpy(mymercury->error_attr,resp.c_str());
						DEBUG_STREAM << "mercurythread::run(): read error: " << resp;
						if(resp != "0")
							abort_sleep(3*((double)mymercury->wr_sleep)/1000);
					}
					abort_sleep(((double)mymercury->wr_sleep)/1000);
				}
			}
			bool read_stat = false;
//#if 0
			if(mymercury->WriteReadMercury(TELL_STATUS, "",resp, true) < 0)
			{
				WARN_STREAM << "mercurythread::run(): exception reading status: '" << resp << "'" << endl;
				ret = -1;
			}		
			else if((mymercury->c_86x_type < C_863_11) && (resp.length() == 19))
			{
				int x[5]={0,0,0,0,0};
				int err;
				if(sscanf(resp.c_str(), "S:%2X %2X %2X %2X %2X %2X",&x[0],&x[1],&x[2], &x[3], &x[4],&err) == 6)
				{
					DEBUG_STREAM << "mercurythread::run(): read status: stat=0x" << hex << x[0] << x[1] << x[2] << x[3] << x[4] << err << endl;
					int j=0;
					for(int i=0; i < STAT_SIZE; i++)
					{
						j = (i < 8) ? 0 : (i < 16) ? 1 : (i < 24) ? 2 : (i < 32) ? 3 : 4; 
						mymercury->stat[i] = x[j] & 0x1 << (i % 8);
						//printf("j=%d - stat[%d]=%d\n",j,i,mymercury->stat[i]);
						
					}
					if(mymercury->c_86x_type == C_863 && mymercury->c_863_10_new)
						mymercury->referenced = mymercury->stat[STAT_BIT_REFERENCED];
				}
				if(err > 0 && err <= MAX_ERRORS)
				{
					strcpy(mymercury->error_attr,errors[err][mymercury->c_86x_type]);		//update only for err > 0 (err != no error)
				}
				read_stat = true;
			}
			//else if((mymercury->c_86x_type == C_863_11) && (resp.length() == 6))
			else if((mymercury->c_86x_type == C_863_11 || mymercury->c_86x_type == C_863_12) && (resp.length() >=7))
			{
				int ax;
				int reg;
				uint32_t xstat;
				//int ret=sscanf(resp.c_str(), "%d %d=%x",&ax, &reg, &xstat);
				if(sscanf(resp.c_str(), "%d %d=%x",&ax, &reg, &xstat) == 3)
				//if(ret == 3)
				{
					DEBUG_STREAM << "mercurythread::run(): read status: stat=0x" << hex << xstat << endl;
				}
				else
				{
				DEBUG_STREAM << "mercurythread::run(): read status: ret="<<ret<<" ax="<<ax<<" reg="<<reg<<" stat=0x" << hex << xstat <<dec<< endl;
				}
				read_stat = true;
				memset(mymercury->stat,0,STAT_SIZE*sizeof(bool));
				mymercury->stat[STAT_BIT_ON_TARGET] = xstat & 0x8000;
				mymercury->stat[STAT_BIT_OFF] = !(xstat & 0x1000);
				mymercury->stat[STAT_POS_LIMIT] = !(xstat & 0x0004);
				mymercury->stat[STAT_NEG_LIMIT] = !(xstat & 0x0001);
				mymercury->stat[STAT_BIT_ON_REFERENCE] = !(xstat & 0x0002);
				mymercury->stat[STAT_BIT_REFERENCING] = (xstat & 0x4000);
			}
			else
				DEBUG_STREAM << "mercurythread::run(): read status: c_86x_type" << (int)mymercury->c_86x_type<<" resp.length()="<<resp.length();
				/*mymercury->stat = stat;
				//mymercury->position_tmp = steps;*/
//#endif
			if(read_stat)
			{
				if(!mymercury->stat[STAT_BIT_ON_TARGET] && ((mymercury->c_86x_type < C_863_11) || !mymercury->stat[STAT_BIT_OFF]))
				{
					if(mymercury->get_state()!=Tango::MOVING)
					{
						mymercury->set_state(Tango::MOVING);
					}
						if(!mymercury->scan_mode && !mymercury->homing)	
							mymercury->set_status("Motor MOVING");
						else if(mymercury->scan_mode)
							mymercury->set_status("Motor SCANNING");
						else if(mymercury->homing || ((mymercury->c_86x_type == C_863_11 || mymercury->c_86x_type == C_863_12) && mymercury->stat[STAT_BIT_REFERENCING]))
							mymercury->set_status("Motor HOMING");
						vector<string> filterable_names;
						vector<double> filterable_values;
						stat_moving = Tango::MOVING;
						//mymercury->push_event("State", filterable_names, filterable_values, &stat_moving);
					//}					
				}
				else //not moving
				{
					Tango::DevState old_state;
					old_state = mymercury->get_state();				
					if(old_state == Tango::MOVING || mymercury->moving)		//moving needed if movement too fast so status has not changed to Tango::MOVING
					{
						struct timeval ts;
						Tango::TimeVal ts_tmp;
						struct timezone tz;	
						gettimeofday(&ts, &tz);
						ts_tmp.tv_sec = ts.tv_sec - 5;	//TODO: now waiting 5 seconds before checking if homing finished
						ts_tmp.tv_usec = ts.tv_usec;
						ts_tmp.tv_nsec = 0;
						if(mymercury->homing == 1 && (mymercury->tv_compare(mymercury->ts_homing,ts_tmp) < 0))
						{
							DEBUG_STREAM << "mercurythread::run(): finished finding edge in homing!" << endl;
							
							if(mymercury->c_86x_type == C_863_11 || mymercury->c_86x_type == C_863_12)
							{
								abort_sleep(((double)mymercury->wr_sleep)/1000);
								if(mymercury->WriteReadMercury(TELL_11_REFERENCED,"",resp, true) < 0)
								{
									WARN_STREAM << "mercurythread::run(): exception reading referenced: '" << resp << "'" << endl;
									ret = -1;
								}
								else if(resp.length() > 2)
								{
									int ax;
									int ref=0;
									if(sscanf(resp.c_str(), "%d=%d",&ax, &ref) == 2)
									{
										if(ref != 1)
										{
											mymercury->referenced = false;
										}
										else
										{
											mymercury->referenced = true;
										}
										DEBUG_STREAM << "mercurythread::run(): read referenced: " << ref << endl;
									}
								}
							}
							
							string resp;
							char temp_c[64];
							double temp_offset;
							temp_offset = mymercury->homeEdgeOffset * mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::homing(): temp_offset=" << temp_offset << endl;
							sprintf(temp_c,"%d",(int)(temp_offset));
							DEBUG_STREAM << "mercurythread::homing(): writing " << temp_c << endl;	
							if(mymercury->WriteReadMercury(CMD_MOV_RELATIVE,temp_c,resp, false) < 0)
							{
								INFO_STREAM << "mercurythread::homing(): error in move relative for homing: " << resp << endl;
								mymercury->homing = 0;
							}				
							mymercury->homing = 2;
							struct timeval ts;
							struct timezone tz;	
							gettimeofday(&ts, &tz);
							mymercury->ts_homing.tv_sec = ts.tv_sec;
							mymercury->ts_homing.tv_usec = ts.tv_usec;
							mymercury->ts_homing.tv_nsec = 0;
							abort_sleep(((double)mymercury->wr_sleep)/1000);		
						}
						else if(mymercury->homing == 2 && (mymercury->tv_compare(mymercury->ts_homing,ts_tmp) < 0))
						{
							DEBUG_STREAM << "mercurythread::run(): finished homing!!" << endl;
							if(mymercury->WriteReadMercury(CMD_SET_HOME,"",resp, false) < 0)
							{
								INFO_STREAM << "mercurythread::homing(): error in set home for homing: " << resp << endl;
							}
							mymercury->homing = 0;
						}
						if((!mymercury->scan_mode || (mymercury->scan_mode && !mymercury->scan_repeat && mymercury->scan_dir > 0)) && !mymercury->homing)
						{
						//mymercury->set_state(mymercury->old_state);
						//mymercury->set_status(mymercury->old_status);
							mymercury->scan_mode = false;	//if !mymercury->scan_repeat and arrived to end_pos, disable scan mode			
							mutex_sleep->lock();
							pausesec_dfl = (int)(((double)(5 * mymercury->th_sleep))/1000);					//poll status slow when not moving
							pausenano_dfl = (int)(((5 * mymercury->th_sleep)-pausesec_dfl*1000)*1000000);
							mymercury->moving = false;
							mutex_sleep->unlock();
							DEBUG_STREAM << "mercurythread::run(): finished moving: pausesec_dfl=" << pausesec_dfl << " pausenano_dfl=" << pausenano_dfl << endl;				
						}
						else/* if(mymercury->scan_mode && mymercury->scan_repeat)*/ if(!mymercury->homing)
						{
							double temp_d;
							char temp_c[32];
							if(mymercury->scan_dir < 0)
							{
								mymercury->scan_dir = 1;
								temp_d = mymercury->scan_end_pos*mymercury->positionRatio;
							}
							else if(mymercury->scan_dir > 0)
							{
								mymercury->scan_dir = -1;
								temp_d = mymercury->scan_start_pos*mymercury->positionRatio;
							}
							sprintf(temp_c,"%d",(int)temp_d);
							if(mymercury->WriteReadMercury(CMD_MOV_ABSOLUTE,temp_c,resp, false) < 0)
							{
								INFO_STREAM << "mercurythread::run(): error moving in scan mode" << endl;
								mymercury->scan_mode = false;
							}
						}
						abort_sleep(((double)mymercury->wr_sleep)/1000);
						//last read of position for this movement
						if(mymercury->WriteReadMercury(TELL_POSITION, "",resp, true) < 0)
						{
							WARN_STREAM << "mercurythread::run(): exception reading position: '" << resp << "'" << endl;
							ret = -1;
						}		
						else if(resp.length() > 2)
						{
							int pos;
							if(sscanf(resp.c_str(), "P:%d",&pos) == 1)
							{
								*(mymercury->attr_Position_read) = (double)(pos) / mymercury->positionRatio;
								DEBUG_STREAM << "mercurythread::run(): read last position: " << pos << endl;
							}
						}
					}
					if(mymercury->stat[STAT_BIT_OFF])
					{
						mymercury->set_state(Tango::OFF);
						mymercury->set_status("Motor loop OFF");				
					}
					else if(mymercury->stat[STAT_BIT_BRAKE])
					{
						mymercury->set_state(Tango::STANDBY);
						mymercury->set_status("Brake ON");				
					}
					else if(!mymercury->referenced && ((mymercury->c_86x_type == C_863_11 || mymercury->c_86x_type == C_863_12) || ( mymercury->c_86x_type == C_863 && mymercury->c_863_10_new)))
					{
						mymercury->set_state(Tango::DISABLE);
						mymercury->set_status("Axis unreferenced");				
					}
					else if(!mymercury->stat[STAT_BIT_OFF] && (!mymercury->scan_mode) && (!mymercury->homing))
					{
						mymercury->set_state(Tango::ON);
						mymercury->set_status("Motor loop ON");				
					}
					else if(!mymercury->scan_mode && !mymercury->homing)		//if I don't know which state to set now, set the old one
					{
						mymercury->set_state(mymercury->get_prev_state());
						mymercury->set_status("Motor NOT moving");				
					}
				}
			}
			//if want to read only when moving uncomment it
	/*		abort_sleep(((double)mymercury->wr_sleep)/1000);
			if(mymercury->get_state() == Tango::MOVING || mymercury->moving)
			{
				abort_sleep(((double)mymercury->wr_sleep)/1000);
				if(mymercury->WriteReadMercury(TELL_POSITION,"",resp, true) < 0)
				{
					WARN_STREAM << "mercurythread::run(): exception reading position: '" << resp << "'" << endl;
					ret = -1;
				}
				
				if(resp.length() > 2)
				{
					int pos;
					sscanf(resp.c_str(), "P:%d",&pos);
					*(mymercury->attr_Position_read) = (double)pos / mymercury->positionRatio;
					DEBUG_STREAM << "mercurythread::run(): read position: " << pos << endl;
				}		
			}*/
			if(mymercury->rtmode)
				continue;
			if((counter % 3) == 0/* && !mymercury->homing*/)
			{
				abort_sleep(((double)mymercury->wr_sleep)/1000);
				if(mymercury->WriteReadMercury(TELL_TARGET_POSITION,"",resp, true) < 0)
				{
					WARN_STREAM << "mercurythread::run(): exception reading target position: '" << resp << "'" << endl;
					ret = -1;
				}
				else if(resp.length() > 2)
				{
					int pos;
					if(mymercury->c_86x_type < C_863_11)
					{
						if(sscanf(resp.c_str(), "T:%d",&pos) == 1)
						{
							*(mymercury->attr_TargetPosition_read) = (double)(pos) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read target position: " << pos << endl;
						}
					}
					else
					{
						int ax;
						float dpos;
						if(sscanf(resp.c_str(), "%d=%f",&ax, &dpos) == 2)
						{
							*(mymercury->attr_TargetPosition_read) = (double)(dpos) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read position: " << dpos << endl;
						}
					}
				}			
			}
			else if((counter % 4) == 1/*  && !mymercury->homing*/)
			{
				abort_sleep(((double)mymercury->wr_sleep)/1000);
				if(mymercury->WriteReadMercury(TELL_PROGRAMMED_VELOCITY,"",resp, true) < 0)		//TODO: velocity or programmed velocity?
				{
					WARN_STREAM << "mercurythread::run(): exception reading speed: '" << resp << "'" << endl;
					ret = -1;
				}
				else if(resp.length() > 2)
				{
					int vel;
					if(mymercury->c_86x_type < C_863_11)
					{
						if(sscanf(resp.c_str(), "Y:%d",&vel) == 1)
						{
							if(!mymercury->scan_mode)
								*(mymercury->attr_Speed_read) = (double)(vel) / mymercury->positionRatio;
							else
								*(mymercury->attr_ScanSpeed_read) = (double)(vel) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read speed: " << vel << endl;
						}
					}
					else
					{
						int ax;
						float dvel;
						if(sscanf(resp.c_str(), "%d=%f",&ax, &dvel) == 2)
						{
							if(!mymercury->scan_mode)
								*(mymercury->attr_Speed_read) = (double)(dvel) / mymercury->positionRatio;
							else
								*(mymercury->attr_ScanSpeed_read) = (double)(dvel) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read speed: " << dvel << endl;
						}
					}
				}			
			}
			else if((counter % 4) == 2/*  && !mymercury->homing*/)
			{
				abort_sleep(((double)mymercury->wr_sleep)/1000);
				if(mymercury->WriteReadMercury(TELL_PROGRAMMED_ACCELERATION,"",resp, true) < 0)
				{
					WARN_STREAM << "mercurythread::run(): exception reading acceleration: '" << resp << "'" << endl;
					ret = -1;
				}
				else if(resp.length() > 2)
				{
					if(mymercury->c_86x_type < C_863_11)
					{
						int acc;
						if(sscanf(resp.c_str(), "L:%d",&acc) == 1)
						{
							*(mymercury->attr_Acceleration_read) = (double)(acc) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read acceleration: " << acc << endl;
						}
					}
					else
					{
						int ax;
						float dacc;
						if(sscanf(resp.c_str(), "%d=%f",&ax, &dacc) == 2)
						{
							*(mymercury->attr_Acceleration_read) = (double)(dacc) / mymercury->positionRatio;
							DEBUG_STREAM << "mercurythread::run(): read acceleration: " << dacc << endl;
						}
					}
				}		
			}
			else if((counter % 4) == 3  && (mymercury->c_86x_type == C_863_11 || mymercury->c_86x_type == C_863_12))
			{
				abort_sleep(((double)mymercury->wr_sleep)/1000);
				if(mymercury->WriteReadMercury(TELL_11_REFERENCED,"",resp, true) < 0)
				{
					WARN_STREAM << "mercurythread::run(): exception reading referenced: '" << resp << "'" << endl;
					ret = -1;
				}
				else if(resp.length() > 2)
				{
					int ax;
					int ref=0;
					if(sscanf(resp.c_str(), "%d=%d",&ax, &ref) == 2)
					{
						if(ref != 1)
						{
							mymercury->referenced = false;
						}
						else
						{
							mymercury->referenced = true;
						}
						DEBUG_STREAM << "mercurythread::run(): read referenced: " << ref << endl;
					}
				}
			}
			if(mymercury->rtmode)
				continue;
//#endif
		}
		if (ret < 0) 
			cnt_err++;
		else
			cnt_err = 0;	
		if (cnt_err > max_cnt_err) {
			pausesec = 5;
			pausenano = 0;
		}
		else {
			pausesec = pausesec_dfl;
			pausenano = pausenano_dfl;
		}

		//DEBUG_STREAM << "mercurythread::run(): going to sleep: pausesec_dfl=" << pausesec_dfl << " pausenano_dfl=" << pausenano_dfl << endl;		
		//omni_thread::sleep(pausesec,pausenano);
		//DEBUG_STREAM << "mercurythread::run(): 222 pausesec=" << pausesec << " pausenano=" << pausenano << " -->" << (double)pausesec + ((double)pausenano)/1000000000 << endl;
		abort_sleep((double)pausesec + ((double)pausenano)/1000000000);
		counter++;	
	}
	INFO_STREAM << "mercurythread::run(): exit!!!" << endl;
}

//+------------------------------------------------------------------
//
//	method:			mercurythread::homing
//
//	description:	Homing: find edge, then move for an offset
//
//-------------------------------------------------------------------
void mercurythread::homing()
{
	string resp;
	char temp_c[64];
	double temp_offset;
	temp_offset = mymercury->homeEdgeOffset * mymercury->positionRatio;
	DEBUG_STREAM << "mercurythread::homing(): temp_offset=" << temp_offset << endl;
	sprintf(temp_c,"%s%d,%s,%s%d,%s,%s",CMD_FIND_EDGE,2,CMD_WAIT_STOP,CMD_MOV_RELATIVE, (int)(temp_offset), CMD_WAIT_STOP,CMD_SET_HOME);
	DEBUG_STREAM << "mercurythread::homing(): writing " << temp_c << endl;	
	if(mymercury->WriteReadMercury(temp_c,"",resp, false) < 0)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "PiMercury::find_edge()", Tango::ERR);
	}
	abort_sleep(((double)mymercury->wr_sleep)/1000);
	//mymercury->homing = false;
}

//+------------------------------------------------------------------
//
//	method:			mercurythread::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void mercurythread::abort_sleep(double time) 
{		
	for (int i = 0; i < (time/0.1); i++) {
		if (mymercury->abortflag)
			break;
		if(mymercury->abort_rt_sleep)
			break;
		omni_thread::sleep(0,100000000);
	}
}


